#!/usr/local/bin/perl -w
#
# job.sssc.pl - convert SSSCs to PIDC format
#
# Istvan Bondar, 1999/02/24
#
# Usage: job.sssc.pl sta phase region
#	where sta is a stationcode,
#             phase is a phasename (Pn,Pg,Sn,Lg),
#             region is a shortcut for a region, like nam or fenno.
#
# Input:
#   input filename: sssc.{stacode}.{phase} e.g. sssc.TXAR.Pn
#   input file contains columns of lat, lon, sssc, modeling error
# Output: TT.{sta}.{phase}.reg.{region} - a PIDC compliant SSSC file 
#
#
use strict;
$|=1;					# autoflush

print "create PIDC-format SSSC files from DetSssc's output\n";
#
# check arguments, provide usage info and terminate if there are
#  incorrect number of arguments
#
if ( $#ARGV != 2) {
   print "Usage: job.sssc sta phase region\n";
   print "   where sta is a stationcode,\n";
   print "         phase is a phasename (Pn,Pg,Sn,Lg),\n";
   print "         region is a shortcut for a region, like nam or fenno.\n\n";
   print "Output: TT.{sta}.{phase}.reg.{region} - a PIDC compliant SSSC file\n";
   print "\n   Example: job.sssc ARCES Sn fenno\n";
   if ( $#ARGV < 2 ) {
      print "\nNot enough arguments, the job is terminated.\n";
   }
   else {
      print "\nToo many arguments, the job is terminated.\n";
   }
}   
my ($sta,$phase,$regio,$fname,$i,$j,$k);
my (@LoL,@lat,@lon);

$sta=$ARGV[0];
$phase=$ARGV[1];
$regio=$ARGV[2];
print "$sta, $phase, $regio \n";
#
# read in input file
#
$fname="sssc." . $sta . "." . $phase;
open(IN,"$fname") || die "Can't open $fname\n"; 
while ( <IN> ) {
   push @LoL, [ split ];
}
close(IN);
#
# get latitudes and longitudes
#
for $i ( 0 .. $#LoL ) {
   $lat[$i]=$LoL[$i][0];
   $lon[$i]=$LoL[$i][1];
}
#
# get list of unique latitudes in descending order
#
$j='91.000';
@lat=grep { $_ ne $j && ($j = $_) } sort { $b <=> $a } @lat;
#
# get list of unique longitudes in ascending order
#
$j='-361.000';
@lon=grep { $_ ne $j && ($j = $_) } sort { $a <=> $b } @lon;
#
# reorder SSSCs so that they start from the NW corner of the rectangle
# (i.e. sorted by descending latitudes and ascending longitudes)
#
@LoL=sort { $b->[0] <=> $a->[0] or $a->[1] <=> $b->[1] } @LoL;
#
# open output file for writing
#
$fname="TT." . $sta . "." . $phase . ".reg." . $regio;
open(OU,">$fname") || die "Can't open $fname\n"; 
#
# write header
#
$j='           iasp91   reg   0.000';
print OU "# Station Phase    Source Region    Velocity Model   Level  StaCor  SedVel MEFac\n"; 
print OU "# ======= ======== ================ ================ =====  ======  ====== =====\n";
if ( $phase =~ /^[pP]/ ) {
   printf OU "  %-7s %-8s %-16s%s   5.800   1.0\n",$sta,$phase,$regio,$j;
}
else {
   printf OU "  %-7s %-8s %-16s%s   3.350   1.0\n",$sta,$phase,$regio,$j;
}
print OU "# Number of Lat/Lon samples, respectively:\n"; 
printf OU "%5d%5d\n",$#lat+1,$#lon+1; 
print OU "# Latitude samples (deg) from north to south:\n"; 
for $i ( 0 .. $#lat ) {
   printf OU "%10.3f",$lat[$i];
}
print OU "\n";
print OU "# Longitude samples (deg) from west to east:\n"; 
for $i ( 0 .. $#lon ) {
   printf OU "%10.3f",
      ($lon[$i]<-180.) ? $lon[$i]+360 : ($lon[$i]>180) ? $lon[$i]-360: $lon[$i];
}
print OU "\n";
#
# write SSSCs
#
print OU "# SSSC's in map view (origin is NW corner):\n"; 
$k=0;
for $i ( 0 .. $#lat ) {
   for $j ( 0 .. $#lon ) {
      printf OU "%8.2f",$LoL[$k++][2];
   }
   print OU "\n";
}
#
# write MEs
#
print OU "# Modeling errors in map view (origin is NW corner):\n"; 
$k=0;
for $i ( 0 .. $#lat ) {
   for $j ( 0 .. $#lon ) {
      printf OU "%8.2f",$LoL[$k++][3];
   }
   print OU "\n";
}
close(OU);
print "Done.\n";
